import { useState } from "react";
import { RiDeleteBinLine } from "react-icons/ri";
import {
  MdExpandLess,
  MdExpandMore,
  MdEdit,
  MdOutlineSave,
  MdOutlineContentCopy,
  MdDownload,
  MdOutlineInsertDriveFile,
  MdOutlineCancel,
} from "react-icons/md";

import Rule from "./Rule";
import RuleInput from "./RuleInput";

const Grammar = ({
  grammar,
  deleteGr,
  editGr,
  addRule,
  duplicateGr,
  deleteRule,
  changeRule,
  getFile,
  changeRoot,
}) => {
  const [more, setMore] = useState(false);
  const [ruleAdd, setRuleAdd] = useState(false);
  const [edit, setEdit] = useState(false);
  const [uri, setUri] = useState("");
  const [newRule, setNewRule] = useState({});
  const [file, setFile] = useState(false);

  function switchInfo() {
    setMore(!more);
  }
  function handleSubmit(e) {
    e.preventDefault();
    if (uri == null || uri.length == 0) {
      return;
    }
    setEdit(false);
    var gr = { uri: uri, id: grammar.id };
    editGr(gr);
  }

  function handleNewRule(e) {
    e.preventDefault();
    if (
      newRule.rightSide == null ||
      newRule.leftSymbol == null ||
      newRule.rightSide.lengh == 0 ||
      newRule.leftSymbol.length == 0
    ) {
      return;
    }
    setRuleAdd(false);
    setNewRule({});
    addRule(newRule, grammar.id);
  }

  function changeRuleAddGrammarId(ruleId, rule) {
    changeRule(ruleId, rule, grammar.id);
  }

  function switchEditmode() {
    if (edit) {
      setRuleAdd(false);
    } else {
      setMore(!edit);
    }

    setEdit(!edit);
  }

  function deleteRuleAddGrammarId(ruleID, root) {
    deleteRule(ruleID, root, grammar.id);
  }

  function changeRootAddGrammarID(ruleId, rule) {
    changeRoot(ruleId, grammar.id, rule);
  }
  return (
    <>
      <div className="grammar">
        <article>
          <h3>
            {grammar.uri} {edit && <MdOutlineSave onClick={handleSubmit} />}{" "}
          </h3>
          {edit && (
            <form onSubmit={(e) => handleSubmit(e)}>
              <input
                onChange={(e) => setUri(e.target.value)}
                type="text"
                className="transparent"
                placeholder="Change uri"
              />
            </form>
          )}
          <p className="rootRuleSmall">
            {grammar.rootRule != undefined &&
              grammar.rootRule.leftSymbol + " = " + grammar.rootRule.rightSide}
          </p>
          <MdOutlineInsertDriveFile onClick={() => setFile(!file)} />{" "}
          {more ? (
            <MdExpandLess onClick={switchInfo} />
          ) : (
            <MdExpandMore onClick={switchInfo} />
          )}
          <MdEdit onClick={switchEditmode} />{" "}
          <MdOutlineContentCopy onClick={() => duplicateGr(grammar.id)} />{" "}
          <RiDeleteBinLine onClick={() => deleteGr(grammar.id)} />
          <br></br>
          {more &&
            grammar.rules.map((rule) => (
              <Rule
                key={rule.id}
                rule={rule}
                deleteRule={deleteRuleAddGrammarId}
                changeRule={changeRuleAddGrammarId}
                masterRoot={grammar.rootRule != null}
                changeRoot={changeRootAddGrammarID}
              />
            ))}
          {edit && (
            <>
              {!ruleAdd ? (
                <>
                  <button onClick={() => setRuleAdd(true)}>ADD RULE</button>{" "}
                </>
              ) : (
                <form onSubmit={handleNewRule}>
                  <RuleInput
                    rule={newRule}
                    setRule={setNewRule}
                    haveRoot={grammar.rootRule != null}
                  />{" "}
                  <MdOutlineSave onClick={handleNewRule} />
                  <MdOutlineCancel onClick={() => setRuleAdd(false)} />
                </form>
              )}
              <br></br>
            </>
          )}
          {file && (
            <>
              {" "}
              <a
                href={
                  "http://localhost:8080/api/grammars/" +
                  grammar.id +
                  "/file/xml/download"
                }
              >
                <MdDownload />
              </a>{" "}
              <button onClick={() => getFile(true, grammar.id)}> xml</button>{" "}
              <button onClick={() => getFile(false, grammar.id)}> jsgf</button>{" "}
              <a
                href={
                  "http://localhost:8080/api/grammars/" +
                  grammar.id +
                  "/file/jsgf/download"
                }
              >
                <MdDownload />
              </a>{" "}
              <br></br>
              <textarea
                className="transparent"
                readOnly={true}
                rows="4"
                cols="50"
                value={grammar.xml}
              />
              <textarea
                className="transparent"
                readOnly={true}
                rows="4"
                cols="50"
                value={grammar.jsgf}
              />
            </>
          )}
        </article>
      </div>{" "}
      <br></br>
    </>
  );
};

export default Grammar;
